<?php
/**
 * Plugin Name: Simple Profile Picture by Corenexis
 * Plugin URI: https://corenexis.com
 * Description: A lightweight, fast plugin that enables users to set custom profile pictures from the WordPress Media Library instead of Gravatar. Developed by corenexis.
 * Version: 2.1.2
 * Author: Corenexis
 * Author URI: https://corenexis.com
 * Text Domain: simple-profile-picture
 * Domain Path: /languages
 * 
 * Copyright (c) 2025 Corenexis. All rights reserved.
 * This code is the intellectual property of Corenexis and may not be
 * distributed, modified or used without explicit permission.
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

// Define plugin constants
define('SPP_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('SPP_PLUGIN_URL', plugin_dir_url(__FILE__));
define('SPP_VERSION', '2.0.2');

/**
 * Main Plugin Class
 */
class Simple_Profile_Picture {

    /**
     * Constructor
     */
    public function __construct() {
        // Register activation, deactivation hooks
        register_activation_hook(__FILE__, array($this, 'activate'));
        register_deactivation_hook(__FILE__, array($this, 'deactivate'));
        
        // Init plugin
        add_action('plugins_loaded', array($this, 'init'));
    }

    /**
     * Initialize plugin
     */
    public function init() {
        // Load text domain for internationalization
        load_plugin_textdomain('simple-profile-picture', false, dirname(plugin_basename(__FILE__)) . '/languages');
        
        // Add admin menu for settings
        add_action('admin_menu', array($this, 'add_admin_menu'));
        
        // Add setting fields to user profile
        add_action('show_user_profile', array($this, 'add_profile_picture_field'));
        add_action('edit_user_profile', array($this, 'add_profile_picture_field'));
        
        // Save profile picture
        add_action('personal_options_update', array($this, 'save_profile_picture'));
        add_action('edit_user_profile_update', array($this, 'save_profile_picture'));
        
        // Add AJAX handler for default avatar
        add_action('wp_ajax_spp_get_default_avatar', array($this, 'ajax_get_default_avatar'));
        
        // Replace Gravatar with custom profile picture
        add_filter('get_avatar', array($this, 'replace_avatar'), 10, 6);
        
        // Replace avatar in admin bar
        add_action('admin_bar_menu', array($this, 'fix_admin_bar_avatar'), 0);
        
        // Enqueue necessary scripts and styles
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
    }

    /**
     * Activation hook
     */
    public function activate() {
        // Set default settings on activation
        $default_settings = array(
            'allowed_roles' => array('administrator', 'editor', 'author')
        );
        
        if (!get_option('spp_settings')) {
            update_option('spp_settings', $default_settings);
        }
    }

    /**
     * Deactivation hook
     */
    public function deactivate() {
        // Nothing specific to do on deactivation
    }

    /**
     * Enqueue scripts and styles for admin
     */
    public function enqueue_admin_scripts($hook) {
        // Only enqueue on profile or user-edit page or our plugin page
        if ($hook === 'profile.php' || $hook === 'user-edit.php' || $hook === 'settings_page_simple-profile-picture') {
            // Enqueue WordPress media scripts
            wp_enqueue_media();
            
            // Enqueue custom script
            wp_enqueue_script(
                'spp-admin-script',
                SPP_PLUGIN_URL . 'assets/js/admin.js',
                array('jquery'),
                SPP_VERSION,
                true
            );
            
            // Localize script
            wp_localize_script(
                'spp-admin-script',
                'sppData',
                array(
                    'title' => __('Select or Upload Profile Picture', 'simple-profile-picture'),
                    'button' => __('Use this image', 'simple-profile-picture'),
                    'security' => wp_create_nonce('spp_nonce'),
                    'ajaxurl' => admin_url('admin-ajax.php')
                )
            );
            
            // Enqueue custom style
            wp_enqueue_style(
                'spp-admin-style',
                SPP_PLUGIN_URL . 'assets/css/admin.css',
                array(),
                SPP_VERSION
            );
        }
    }

    /**
     * Add settings menu page
     */
    public function add_admin_menu() {
        add_options_page(
            __('Profile Picture Settings', 'simple-profile-picture'),
            __('Profile Picture', 'simple-profile-picture'),
            'manage_options',
            'simple-profile-picture',
            array($this, 'settings_page')
        );
        
        // Register settings
        register_setting('spp_settings_group', 'spp_settings', array($this, 'sanitize_settings'));
    }

    /**
     * Settings page callback
     */
    public function settings_page() {
        // Get current settings
        $settings = get_option('spp_settings', array());
        
        // Get available roles
        $roles = wp_roles()->get_names();
        
        // Get current user
        $current_user = wp_get_current_user();
        
        // Get current profile picture ID
        $profile_pic_id = get_user_meta($current_user->ID, 'spp_profile_picture_id', true);
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            
            <div class="spp-tabs">
                <button class="spp-tab-link active" data-tab="roles"><?php _e('Role Settings', 'simple-profile-picture'); ?></button>
                <button class="spp-tab-link" data-tab="my-picture"><?php _e('My Profile Picture', 'simple-profile-picture'); ?></button>
            </div>
            
            <div class="spp-tab-content" id="spp-tab-roles" style="display:block;">
                <form method="post" action="options.php">
                    <?php settings_fields('spp_settings_group'); ?>
                    <?php do_settings_sections('spp_settings_group'); ?>
                    
                    <table class="form-table">
                        <tr valign="top">
                            <th scope="row"><?php _e('Allowed User Roles', 'simple-profile-picture'); ?></th>
                            <td>
                                <?php foreach ($roles as $role_slug => $role_name) : ?>
                                    <label>
                                        <input type="checkbox" name="spp_settings[allowed_roles][]" value="<?php echo esc_attr($role_slug); ?>" 
                                            <?php checked(in_array($role_slug, isset($settings['allowed_roles']) ? $settings['allowed_roles'] : array())); ?>>
                                        <?php echo esc_html($role_name); ?>
                                    </label><br>
                                <?php endforeach; ?>
                                <p class="description"><?php _e('Select which user roles can set custom profile pictures.', 'simple-profile-picture'); ?></p>
                            </td>
                        </tr>
                    </table>
                    
                    <?php submit_button(); ?>
                </form>
                
                <div class="spp-support">
                    <p>
                        <a href="https://corenexis.com" class="button" target="_blank">
                            <?php _e('Visit Corenexis Website', 'simple-profile-picture'); ?>
                        </a>
                    </p>
                </div>
            </div>
            
            <div class="spp-tab-content" id="spp-tab-my-picture" style="display:none;">
                <h2><?php _e('Your Profile Picture', 'simple-profile-picture'); ?></h2>
                
                <?php if ($this->user_can_set_profile_picture($current_user->ID)) : ?>
                    <div class="spp-profile-form">
                        <div class="spp-profile-preview">
                            <h3><?php _e('Current Picture', 'simple-profile-picture'); ?></h3>
                            <div class="spp-preview-wrapper">
                                <div class="spp-preview">
                                    <?php if ($profile_pic_id) : ?>
                                        <?php echo wp_get_attachment_image($profile_pic_id, 'thumbnail'); ?>
                                    <?php else : ?>
                                        <?php echo get_avatar($current_user->ID, 96); ?>
                                    <?php endif; ?>
                                </div>
                            </div>
                        </div>
                        
                        <div class="spp-profile-actions">
                            <input type="hidden" name="spp_profile_picture_id" id="spp_profile_picture_id" value="<?php echo esc_attr($profile_pic_id); ?>">
                            
                            <p>
                                <button type="button" class="button button-primary spp-upload-button" id="spp_upload_button">
                                    <?php _e('Select Image', 'simple-profile-picture'); ?>
                                </button>
                                
                                <?php if ($profile_pic_id) : ?>
                                    <button type="button" class="button spp-remove-button" id="spp_remove_button">
                                        <?php _e('Remove Image', 'simple-profile-picture'); ?>
                                    </button>
                                <?php endif; ?>
                            </p>
                            
                            <p class="description">
                                <?php _e('Upload or select an image from the media library to use as your profile picture.', 'simple-profile-picture'); ?>
                            </p>
                            
                            <?php wp_nonce_field('spp_profile_picture', 'spp_nonce'); ?>
                            
                            <p>
                                <button type="button" class="button button-primary" id="spp_save_settings_profile">
                                    <?php _e('Save Profile Picture', 'simple-profile-picture'); ?>
                                </button>
                            </p>
                            
                            <div id="spp-save-message" style="display:none;"></div>
                        </div>
                    </div>
                <?php else : ?>
                    <p><?php _e('You do not have permission to set a custom profile picture.', 'simple-profile-picture'); ?></p>
                <?php endif; ?>
            </div>
        </div>
        
        <script>
        jQuery(document).ready(function($) {
            // Tab functionality
            $('.spp-tab-link').on('click', function(e) {
                e.preventDefault();
                var tabId = $(this).data('tab');
                
                // Hide all tab contents and deactivate all tab links
                $('.spp-tab-content').hide();
                $('.spp-tab-link').removeClass('active');
                
                // Show the selected tab content and activate the tab link
                $('#spp-tab-' + tabId).show();
                $(this).addClass('active');
            });
            
            // Save profile picture from settings page
            $('#spp_save_settings_profile').on('click', function() {
                var button = $(this);
                var message = $('#spp-save-message');
                
                // Disable button to prevent multiple submissions
                button.prop('disabled', true);
                
                // Show loading message
                message.html('<p class="spp-loading"><?php _e('Saving...', 'simple-profile-picture'); ?></p>').show();
                
                // Send AJAX request
                $.ajax({
                    url: sppData.ajaxurl,
                    type: 'POST',
                    data: {
                        action: 'spp_save_profile_picture',
                        profile_picture_id: $('#spp_profile_picture_id').val(),
                        security: $('#spp_nonce').val()
                    },
                    success: function(response) {
                        if (response.success) {
                            message.html('<p class="spp-success">' + response.data + '</p>');
                        } else {
                            message.html('<p class="spp-error">' + response.data + '</p>');
                        }
                        
                        // Re-enable button
                        button.prop('disabled', false);
                        
                        // Hide message after 3 seconds
                        setTimeout(function() {
                            message.fadeOut();
                        }, 3000);
                    },
                    error: function() {
                        message.html('<p class="spp-error"><?php _e('An error occurred. Please try again.', 'simple-profile-picture'); ?></p>');
                        
                        // Re-enable button
                        button.prop('disabled', false);
                    }
                });
            });
        });
        </script>
        <?php
    }

    /**
     * Sanitize settings
     */
    public function sanitize_settings($input) {
        $sanitized_input = array();
        
        if (isset($input['allowed_roles']) && is_array($input['allowed_roles'])) {
            $sanitized_input['allowed_roles'] = array_map('sanitize_text_field', $input['allowed_roles']);
        } else {
            $sanitized_input['allowed_roles'] = array();
        }
        
        return $sanitized_input;
    }

    /**
     * AJAX handler for saving profile picture from settings page
     */
    public function ajax_save_profile_picture() {
        // Check nonce
        if (!isset($_POST['security']) || !wp_verify_nonce($_POST['security'], 'spp_profile_picture')) {
            wp_send_json_error(__('Security check failed.', 'simple-profile-picture'));
        }
        
        // Get current user
        $current_user = wp_get_current_user();
        
        // Check if user has permission
        if (!$this->user_can_set_profile_picture($current_user->ID)) {
            wp_send_json_error(__('You do not have permission to set a profile picture.', 'simple-profile-picture'));
        }
        
        // Get profile picture ID
        $profile_pic_id = isset($_POST['profile_picture_id']) ? absint($_POST['profile_picture_id']) : 0;
        
        if ($profile_pic_id > 0) {
            // Make sure the attachment exists
            if (get_post_type($profile_pic_id) === 'attachment') {
                update_user_meta($current_user->ID, 'spp_profile_picture_id', $profile_pic_id);
                
                // Generate and save a 64x64 version for admin bar
                $this->generate_admin_bar_image($profile_pic_id, $current_user->ID);
                
                wp_send_json_success(__('Profile picture updated successfully.', 'simple-profile-picture'));
            } else {
                wp_send_json_error(__('Invalid image selected.', 'simple-profile-picture'));
            }
        } else {
            delete_user_meta($current_user->ID, 'spp_profile_picture_id');
            delete_user_meta($current_user->ID, 'spp_admin_bar_picture_url');
            wp_send_json_success(__('Profile picture removed.', 'simple-profile-picture'));
        }
    }
    
    /**
     * Generate a properly sized admin bar avatar and save the URL
     */
    public function generate_admin_bar_image($attachment_id, $user_id) {
        // Get the attachment path
        $attachment_path = get_attached_file($attachment_id);
        
        if (!$attachment_path) {
            return;
        }
        
        // Check if we need to create a custom size
        $metadata = wp_get_attachment_metadata($attachment_id);
        $custom_size_exists = false;
        
        if (isset($metadata['sizes']) && isset($metadata['sizes']['spp_admin_avatar'])) {
            $custom_size_exists = true;
        }
        
        // If the custom size doesn't exist yet, create it
        if (!$custom_size_exists) {
            // Add our custom image size if not already registered
            if (!has_image_size('spp_admin_avatar')) {
                add_image_size('spp_admin_avatar', 64, 64, true);
            }
            
            // Generate the thumbnail
            $editor = wp_get_image_editor($attachment_path);
            
            if (!is_wp_error($editor)) {
                $editor->resize(64, 64, true);
                $filename = $editor->generate_filename('spp_admin_avatar');
                $editor->save($filename);
                
                // Update attachment metadata
                $metadata = wp_get_attachment_metadata($attachment_id);
                $upload_dir = wp_upload_dir();
                
                // Get file info
                $info = pathinfo($filename);
                $rel_path = str_replace($upload_dir['basedir'] . '/', '', $filename);
                
                // Add size to metadata
                $metadata['sizes']['spp_admin_avatar'] = array(
                    'file' => basename($filename),
                    'width' => 64,
                    'height' => 64,
                    'mime-type' => get_post_mime_type($attachment_id)
                );
                
                // Update attachment metadata
                wp_update_attachment_metadata($attachment_id, $metadata);
            }
        }
        
        // Get and save the URL of the custom size
        $image_url = wp_get_attachment_image_url($attachment_id, 'spp_admin_avatar');
        
        if ($image_url) {
            update_user_meta($user_id, 'spp_admin_bar_picture_url', $image_url);
        }
    }

    /**
     * AJAX handler for getting default avatar
     */
    public function ajax_get_default_avatar() {
        // Check nonce
        if (!isset($_POST['security']) || !wp_verify_nonce($_POST['security'], 'spp_nonce')) {
            wp_send_json_error();
        }
        
        // Get user ID
        $user_id = isset($_POST['user_id']) ? absint($_POST['user_id']) : 0;
        
        // Get default avatar
        $avatar = get_avatar($user_id, 96);
        
        wp_send_json_success($avatar);
    }

    /**
     * Check if user has permission to set profile picture
     */
    public function user_can_set_profile_picture($user_id) {
        $settings = get_option('spp_settings', array());
        $allowed_roles = isset($settings['allowed_roles']) ? $settings['allowed_roles'] : array();
        
        // Admin can always set profile picture
        if (current_user_can('manage_options')) {
            return true;
        }
        
        // Get user data
        $user = get_userdata($user_id);
        
        if (!$user) {
            return false;
        }
        
        // Check if user has any of the allowed roles
        foreach ($allowed_roles as $role) {
            if (in_array($role, (array) $user->roles)) {
                return true;
            }
        }
        
        return false;
    }

    /**
     * Add profile picture field to user profile
     */
    public function add_profile_picture_field($user) {
        // Check if user has permission
        if (!$this->user_can_set_profile_picture($user->ID) && !current_user_can('manage_options')) {
            return;
        }
        
        // Get current profile picture ID
        $profile_pic_id = get_user_meta($user->ID, 'spp_profile_picture_id', true);
        ?>
        <h3><?php _e('Profile Picture', 'simple-profile-picture'); ?></h3>
        
        <table class="form-table">
            <tr>
                <th><label for="spp_profile_picture"><?php _e('Custom Profile Picture', 'simple-profile-picture'); ?></label></th>
                <td>
                    <div class="spp-preview-wrapper">
                        <div class="spp-preview">
                            <?php if ($profile_pic_id) : ?>
                                <?php echo wp_get_attachment_image($profile_pic_id, 'thumbnail'); ?>
                            <?php else : ?>
                                <?php echo get_avatar($user->ID, 96); ?>
                            <?php endif; ?>
                        </div>
                        
                        <input type="hidden" name="spp_profile_picture_id" id="spp_profile_picture_id" value="<?php echo esc_attr($profile_pic_id); ?>">
                        
                        <p>
                            <button type="button" class="button spp-upload-button" id="spp_upload_button">
                                <?php _e('Select Image', 'simple-profile-picture'); ?>
                            </button>
                            
                            <?php if ($profile_pic_id) : ?>
                                <button type="button" class="button spp-remove-button" id="spp_remove_button">
                                    <?php _e('Remove Image', 'simple-profile-picture'); ?>
                                </button>
                            <?php endif; ?>
                        </p>
                        
                        <p class="description">
                            <?php _e('Upload or select an image from the media library to use as your profile picture.', 'simple-profile-picture'); ?>
                        </p>
                    </div>
                    
                    <?php wp_nonce_field('spp_profile_picture', 'spp_nonce'); ?>
                </td>
            </tr>
        </table>
        <?php
    }

    /**
     * Save profile picture
     */
    public function save_profile_picture($user_id) {
        // Check if current user can edit this user
        if (!current_user_can('edit_user', $user_id)) {
            return false;
        }
        
        // Verify nonce
        if (!isset($_POST['spp_nonce']) || !wp_verify_nonce($_POST['spp_nonce'], 'spp_profile_picture')) {
            return false;
        }
        
        // Check if user has permission to set profile picture
        if (!$this->user_can_set_profile_picture($user_id) && !current_user_can('manage_options')) {
            return false;
        }
        
        // Save or delete the profile picture ID
        if (isset($_POST['spp_profile_picture_id'])) {
            $profile_pic_id = absint($_POST['spp_profile_picture_id']);
            
            if ($profile_pic_id > 0) {
                // Make sure the attachment exists
                if (get_post_type($profile_pic_id) === 'attachment') {
                    update_user_meta($user_id, 'spp_profile_picture_id', $profile_pic_id);
                    
                    // Generate and save a 64x64 version for admin bar
                    $this->generate_admin_bar_image($profile_pic_id, $user_id);
                }
            } else {
                delete_user_meta($user_id, 'spp_profile_picture_id');
                delete_user_meta($user_id, 'spp_admin_bar_picture_url');
            }
        }
    }

    /**
     * Fix admin bar avatar by modifying the HTML before it's rendered
     */
    public function fix_admin_bar_avatar($wp_admin_bar) {
        // Check if user is logged in
        if (!is_user_logged_in()) {
            return;
        }
        
        // Get current user
        $current_user = wp_get_current_user();
        
        // Get custom admin bar picture URL
        $admin_bar_picture_url = get_user_meta($current_user->ID, 'spp_admin_bar_picture_url', true);
        
        // If no custom URL but we have a profile picture ID, generate one
        if (!$admin_bar_picture_url) {
            $profile_pic_id = get_user_meta($current_user->ID, 'spp_profile_picture_id', true);
            
            if ($profile_pic_id) {
                $this->generate_admin_bar_image($profile_pic_id, $current_user->ID);
                $admin_bar_picture_url = get_user_meta($current_user->ID, 'spp_admin_bar_picture_url', true);
            }
        }
        
        // If we have a custom URL, update the admin bar HTML
        if ($admin_bar_picture_url) {
            // Remove the default user info node
            $wp_admin_bar->remove_node('my-account');
            
            // Add our custom node with fixed image size
            $user_info = get_avatar($current_user->ID, 64, '', '', array('force_display' => true));
            
            // Replace the avatar URL with our custom one
            $user_info = preg_replace('/src="[^"]*"/i', 'src="' . esc_url($admin_bar_picture_url) . '"', $user_info);
            
            // Build the user info
            $howdy = sprintf(__('Howdy, %s'), '<span class="display-name">' . $current_user->display_name . '</span>');
            $user_info .= '<span class="display-name">' . $current_user->display_name . '</span>';
            
            if ($current_user->display_name !== $current_user->user_login) {
                $user_info .= '<span class="username">' . $current_user->user_login . '</span>';
            }
            
            $user_info .= '<span class="display-name edit-profile">' . __('Edit Profile') . '</span>';
            
            // Add our custom user info node back
            $wp_admin_bar->add_node(array(
                'id' => 'my-account',
                'parent' => 'top-secondary',
                'title' => $user_info,
                'href' => get_edit_profile_url($current_user->ID),
                'meta' => array(
                    'class' => 'with-avatar',
                    'title' => __('My Account'),
                ),
            ));
        }
    }

    /**
     * Replace Gravatar with custom profile picture
     */
    public function replace_avatar($avatar, $id_or_email, $size, $default, $alt, $args) {
        $user_id = 0;
        
        // Get user ID based on the id_or_email parameter
        if (is_numeric($id_or_email)) {
            $user_id = (int) $id_or_email;
        } elseif (is_string($id_or_email) && is_email($id_or_email)) {
            $user = get_user_by('email', $id_or_email);
            if ($user) {
                $user_id = $user->ID;
            }
        } elseif (is_object($id_or_email) && isset($id_or_email->user_id)) {
            $user_id = (int) $id_or_email->user_id;
        }
        
        if ($user_id === 0) {
            return $avatar;
        }
        
        // Get profile picture ID
        $profile_pic_id = get_user_meta($user_id, 'spp_profile_picture_id', true);
        
        if ($profile_pic_id) {
            // For admin bar, use the special optimized size
            if ($size == 64 && is_admin_bar_showing()) {
                $admin_bar_picture_url = get_user_meta($user_id, 'spp_admin_bar_picture_url', true);
                
                if (!$admin_bar_picture_url) {
                    $this->generate_admin_bar_image($profile_pic_id, $user_id);
                    $admin_bar_picture_url = get_user_meta($user_id, 'spp_admin_bar_picture_url', true);
                }
                
                if ($admin_bar_picture_url) {
                    // Build the img tag
                    $avatar = sprintf(
                        '<img alt="%s" src="%s" class="%s" height="%s" width="%s" %s>',
                        esc_attr($alt),
                        esc_url($admin_bar_picture_url),
                        esc_attr($args['class']),
                        esc_attr($args['height']),
                        esc_attr($args['width']),
                        $args['extra_attr']
                    );
                    
                    return $avatar;
                }
            }
            
            // For other contexts, use the regular thumbnail
            $image_url = wp_get_attachment_image_url($profile_pic_id, 'thumbnail');
            
            if ($image_url) {
                // Build the img tag
                $avatar = sprintf(
                    '<img alt="%s" src="%s" class="%s" height="%s" width="%s" %s>',
                    esc_attr($alt),
                    esc_url($image_url),
                    esc_attr($args['class']),
                    esc_attr($args['height']),
                    esc_attr($args['width']),
                    $args['extra_attr']
                );
            }
        }
        
        return $avatar;
    }
}

// Initialize the plugin
$simple_profile_picture = new Simple_Profile_Picture();

// Add AJAX actions
add_action('wp_ajax_spp_save_profile_picture', array($simple_profile_picture, 'ajax_save_profile_picture'));
